﻿//
//
//
// WhereClause
//
//
//
function WhereClause(table, index, expressionBuilder) {
    /// <param name="table" type="String"></param>
    /// <param name="index" type="String" optional="true"></param>
    /// <param name="expressionBuilder" type="Array" optional="true"></param>
    this._ctx = {
        table: table,
        index: index || ":id",
        eb: expressionBuilder 
    }
}

extend(WhereClause.prototype, function () {

    // WhereClause private methods
    function createExpression(ctx, op, value) {
        var expr = [ctx.index, op, value];
        if (ctx.eb) {
            ctx.eb.push(expr);
            expr = ctx.eb;
        }
        return new Expression(new DbQuery(ctx.table, expr));
    }


    //
    // WhereClause public methods
    //
    return {
        between: function (lower, upper, includeLower, includeUpper) {
            /// <summary>
            ///     Filter out records whose where-field lays between given lower and upper values. Applies to Strings, Numbers and Dates.
            /// </summary>
            /// <param name="lower"></param>
            /// <param name="upper"></param>
            /// <param name="includeLower" optional="true">Whether items that equals lower should be included. Default true.</param>
            /// <param name="includeUpper" optional="true">Whether items that equals upper should be included. Default false.</param>
            /// <returns type="Collection"></returns>
            // TODO: fortsätt i detta stuk och sedan implenmentera bakomlöiggande och exekverande funkt...
            return createExpression(this._ctx, "between", [lower, upper, includeLower, includeUpper]);
            var ctx = this._ctx;
            return new this._ctx.collClass({
                tableName: ctx.table.name,
                expr: [ctx.index, "btwn", [lower, upper, includeLower, includeUpper]]
            });
            includeLower = includeLower !== false;   // Default to true
            includeUpper = includeUpper === true;    // Default to false
            if ((lower > upper) ||
                (lower == upper && (includeLower || includeUpper) && !(includeLower && includeUpper)))
                return new this._ctx.collClass(this, IDBKeyRange.only(lower)).limit(0); // Workaround for idiotic W3C Specification that DataError must be thrown if lower > upper. The natural result would be to return an empty collection.
            return new this._ctx.collClass(this, IDBKeyRange.bound(lower, upper, !includeLower, !includeUpper));
        },
        equals: function (value) {
            return new this._ctx.collClass(this, IDBKeyRange.only(value));
        },
        above: function (value) {
            return new this._ctx.collClass(this, IDBKeyRange.lowerBound(value, true));
        },
        aboveOrEqual: function (value) {
            return new this._ctx.collClass(this, IDBKeyRange.lowerBound(value));
        },
        below: function (value) {
            return new this._ctx.collClass(this, IDBKeyRange.upperBound(value, true));
        },
        belowOrEqual: function (value) {
            return new this._ctx.collClass(this, IDBKeyRange.upperBound(value));
        },
        startsWith: function (str) {
            /// <param name="str" type="String"></param>
            if (typeof str != 'string') return fail(new this._ctx.collClass(this), new TypeError("String expected"));
            return this.between(str, str + String.fromCharCode(65535), true, true);
        },
        startsWithIgnoreCase: function (str) {
            /// <param name="str" type="String"></param>
            if (typeof str != 'string') return fail(new this._ctx.collClass(this), new TypeError("String expected"));
            if (str === "") return this.startsWith(str);
            var c = new this._ctx.collClass(this, IDBKeyRange.bound(str.toUpperCase(), str.toLowerCase() + String.fromCharCode(65535)));
            var flt = createIgnoreCaseFilter(c, function (a, b) { return a.indexOf(b) === 0; }, str);
            c._addFilter(function (reversed) {
                if (reversed)
                    throw new Error("reverse() not supported with .startsWithIgnoreCase()");
                return flt;
            }, { match: true });
            return c;
        },
        equalsIgnoreCase: function (str) {
            /// <param name="str" type="String"></param>
            if (typeof str != 'string') return fail(new this._ctx.collClass(this), new TypeError("String expected"));
            var c = new this._ctx.collClass(this, IDBKeyRange.bound(str.toUpperCase(), str.toLowerCase()));
            c._addFilter(createIgnoreCaseFilter(c, function (a, b) { return a === b; }, str), { match: true });
            return c;
        },
        anyOf: function (valueArray) {
            var sortedSet = getSortedSet(arguments);
            if (sortedSet.length === 0) return new this._ctx.collClass(this, IDBKeyRange.only("")).limit(0); // Return an empty collection.
            var c = new this._ctx.collClass(this, IDBKeyRange.bound(sortedSet[0], sortedSet[sortedSet.length - 1]));

            var i = 0;
            c._addFilter(function (reversed) {
                var sorter = ascending,
                    set = sortedSet;

                if (reversed) {
                    sorter = descending;
                    set = set.slice().sort(descending);
                }
                return function (cursor, value, it, up) {
                    var key = cursor.key;
                    while (sorter(key, set[i]) > 0) {
                        // The cursor has passed beyond this key. Check next.
                        ++i;
                        if (i === set.length) {
                            // There is no next. Stop searching.
                            it.resolve();
                            return;
                        }
                    }
                    if (key === set[i]) {
                        // The current cursor value should be included and we should continue a single step in case next item has the same key or possibly our next key in set.
                        up(cursor, value, it, up.up);
                    } else {
                        // cursor.key not yet at set[i]. Forward cursor to the next key to hunt for.
                        cursor.continue(set[i]);
                    }
                };
            }, { match: true });
            return c;
        }
    };
});
